package org.drools.reproducer;

import org.junit.Test;
import org.kie.api.KieServices;
import org.kie.api.builder.KieBuilder;
import org.kie.api.builder.KieFileSystem;
import org.kie.api.builder.KieModule;
import org.kie.api.builder.KieRepository;
import org.kie.api.builder.Message;
import org.kie.api.builder.ReleaseId;
import org.kie.api.runtime.KieContainer;

public class InConstraintForallTest {

    private static final String RULES_DRL = "package org.drools.reproducer\n" +
            "\n" +
            "rule \"in constraint\"\n" +
            "when forall(Pojo(y==42, x in (1,2,3)))\n" +
            "then\n" +
            "end\n";

    private static final String RULES_SELF_JOIN_DRL = "package org.drools.reproducer\n" +
            "\n" +
            "rule \"in constraint\"\n" +
            "when forall(p : Pojo()" +
            "            Pojo(this==p, y==42, x in (1,2,3)))\n" +
            "then\n" +
            "end\n";

    @Test
    public void testInConstraintForall() {
        KieServices ks = KieServices.Factory.get();
        KieRepository kr = ks.getRepository();

        KieModule kModule = buildKieModule("in-constraint-forall","0.0.1", RULES_DRL);

        KieContainer kContainer = ks.newKieContainer(kModule.getReleaseId());

        kr.removeKieModule(kModule.getReleaseId());

    }

    @Test
    public void testInConstraintForallSelfJoin() {
        KieServices ks = KieServices.Factory.get();
        KieRepository kr = ks.getRepository();

        KieModule kModule = buildKieModule("in-constraint-forall-selfjoin","0.0.1", RULES_SELF_JOIN_DRL);

        KieContainer kContainer = ks.newKieContainer(kModule.getReleaseId());

        kr.removeKieModule(kModule.getReleaseId());
    }

    private static KieModule buildKieModule(String name, String version, String rules) {
        KieServices ks = KieServices.Factory.get();
        KieFileSystem kfs = ks.newKieFileSystem();

        ReleaseId rid = ks.newReleaseId("org.drools.reproducer", name, version);
        kfs.generateAndWritePomXML(rid);

        kfs.write("src/main/resources/rules.drl", rules);

        KieBuilder kb = ks.newKieBuilder(kfs);
        kb.buildAll(); // kieModule is automatically deployed to KieRepository if successfully built.
        if (kb.getResults().hasMessages(Message.Level.ERROR)) {
            throw new RuntimeException("Build Errors:\n" + kb.getResults().toString());
        }

        return kb.getKieModule();
    }


}
